@@ -46,29 +46,27 @@
 #' @export
 
 
-NNS.ANOVA <- function(control,
-                     treatment,
-                     confidence.interval = 0.95,
-                     tails = "Both",
-                     pairwise = FALSE,
-                     plot = TRUE,
-                     robust = FALSE){
-
+NNS.ANOVA <- function(
+    control,
+    treatment,
+    confidence.interval = 0.95,
+    tails = "Both",
+    pairwise = FALSE,
+    plot = TRUE,
+    robust = FALSE
+){
     tails <- tolower(tails)
-
-    if(!any(tails%in%c("left","right","both"))) stop("Please select tails from 'left', 'right', or 'both'")
-
-    if(missing(treatment)){
-        n <- ncol(control)
-        if(is.null(n)) stop("supply both 'control' and 'treatment' or a matrix-like 'control'")
-        if(n == 1) stop("supply both 'control' and 'treatment' or a matrix-like 'control'")
-        if(n >= 2){
-            if(any(class(control)%in%c("tbl","data.table"))) A <- as.data.frame(control)
+    if(!any(tails%in%c("left","right","both"))){
+        stop("Please select tails from 'left', 'right', or 'both'")
+    }
+    if(!missing(treatment)){
+        # with treatment
+        if(any(class(control)%in%c("tbl","data.table"))){
+            control <- as.vector(unlist(control))
+        }
+        if(any(class(treatment)%in%c("tbl","data.table"))){
+            treatment <- as.vector(unlist(treatment))
         }
-    } else {
-        if(any(class(control)%in%c("tbl","data.table"))) control <- as.vector(unlist(control))
-        if(any(class(treatment)%in%c("tbl","data.table"))) treatment <- as.vector(unlist(treatment))
-
         if(robust){
             treatment_p <- replicate(100, sample.int(length(treatment), replace = TRUE))
             treatment_matrix <- matrix(treatment[treatment_p], ncol = dim(treatment_p)[2], byrow = F)
@@ -76,13 +74,13 @@ NNS.ANOVA <- function(control,
             control_matrix <- matrix(control[treatment_p], ncol = dim(treatment_p)[2], byrow = F)
             full_matrix <- cbind(control, control_matrix, treatment, treatment_matrix)
 
-            nns.certainties <- sapply(1:ncol(control_matrix), function(g) NNS.ANOVA.bin(control_matrix[,g], treatment_matrix[,g], plot = FALSE)$Certainty)
-
+            nns.certainties <- sapply(
+                1:ncol(control_matrix), 
+                function(g) NNS.ANOVA.bin(control_matrix[,g], treatment_matrix[,g], plot = FALSE)$Certainty
+            )
             cer_lower_CI <- LPM.VaR(.025, 1, nns.certainties[-1])
             cer_upper_CI <- UPM.VaR(.025, 1, nns.certainties[-1])
-
             robust_estimate <- gravity(nns.certainties)
-
             original.par <- par(no.readonly = TRUE)
             par(mfrow = c(1, 2))
             hist(nns.certainties, main = "NNS Certainty")
@@ -90,82 +88,116 @@ NNS.ANOVA <- function(control,
             mtext("Robust Certainty Estimate", side = 3, col = "red", at = robust_estimate)
             abline(v =  cer_lower_CI, col = "red", lwd = 2, lty = 3)
             abline(v =  cer_upper_CI , col = "red", lwd = 2, lty = 3)
-
-
-
-            return(list(NNS.ANOVA.bin(control, treatment, confidence.interval = confidence.interval, plot = plot, tails = tails, par = original.par),
-                        "Robust Certainty Estimate" = robust_estimate,
-                        "95% CI" = c(cer_lower_CI, cer_upper_CI)))
-
-        } else {
-            return(NNS.ANOVA.bin(control, treatment, confidence.interval = confidence.interval, plot = plot, tails = tails))
+            return(
+                list(
+                    NNS.ANOVA.bin(
+                        control, 
+                        treatment, 
+                        confidence.interval = confidence.interval, 
+                        plot = plot, 
+                        tails = tails, 
+                        par = original.par
+                    ),
+                    "Robust Certainty Estimate" = robust_estimate,
+                    "95% CI" = c(cer_lower_CI, cer_upper_CI)
+                )
+            )
         }
+        return(
+            NNS.ANOVA.bin(
+                control, 
+                treatment, 
+                confidence.interval = confidence.interval, 
+                plot = plot, 
+                tails = tails
+            )
+        )
     }
-
-
-    mean.of.means <- mean(colMeans(A))
-    n <- ncol(A)
-
-    if(!pairwise){
-    #Continuous CDF for each variable from Mean of Means
-        LPM_ratio <- sapply(1 : ncol(A), function(b) LPM.ratio(1, mean.of.means, A[ , b]))
-
-        lower.25.target <- mean(sapply(1:n, function(i) LPM.VaR(.25,1,A[,i])))
-        upper.25.target <- mean(sapply(1:n, function(i) UPM.VaR(.25,1,A[,i])))
-        lower.125.target <- mean(sapply(1:n, function(i) LPM.VaR(.125,1,A[,i])))
-        upper.125.target <- mean(sapply(1:n, function(i) UPM.VaR(.125,1,A[,i])))
-
-
-        n <- ncol(A)
-        raw.certainties <- list(n - 1)
-
-        for(i in 1:(n - 1)){
-          raw.certainties[[i]] <- sapply((i + 1) : n, function(b) NNS.ANOVA.bin(A[ , i],A[ , b],
-                                                                                mean.of.means = mean.of.means,
-                                                                                upper.25.target = upper.25.target,
-                                                                                lower.25.target = lower.25.target,
-                                                                                upper.125.target = upper.125.target,
-                                                                                lower.125.target = lower.125.target,
-                                                                                plot = FALSE)$Certainty)
+    # without treatment
+    n <- ncol(control)
+    if(is.null(n)){
+        stop("supply both 'control' and 'treatment' or a matrix-like 'control'")
+    }
+    if(n == 1){
+        stop("supply both 'control' and 'treatment' or a matrix-like 'control'")
+    }
+    if(n >= 2){
+        if(any(class(control) %in% c("tbl","data.table"))){
+            A <- as.data.frame(control)
+        }else{
+            A <- control
         }
-
-
-  #Certainty associated with samples
+    }else{
+        A <- control
+    }
+    mean.of.means <- mean(colMeans(A))
+    if(!pairwise){
+        #Continuous CDF for each variable from Mean of Means
+        LPM_ratio        <-      sapply(1:n, function(b) LPM.ratio(1, mean.of.means, A[ , b]))
+        lower.25.target  <- mean(sapply(1:n, function(i) LPM.VaR(.25,  1, A[,i])))
+        upper.25.target  <- mean(sapply(1:n, function(i) UPM.VaR(.25,  1, A[,i])))
+        lower.125.target <- mean(sapply(1:n, function(i) LPM.VaR(.125, 1, A[,i])))
+        upper.125.target <- mean(sapply(1:n, function(i) UPM.VaR(.125, 1, A[,i])))
+        raw.certainties <- list(n - 1)
+        for(i in 1:(n - 1)){
+          raw.certainties[[i]] <- sapply(
+            (i + 1) : n, 
+            function(b) NNS.ANOVA.bin(
+                A[ , i],
+                A[ , b],
+                mean.of.means = mean.of.means,
+                upper.25.target = upper.25.target,
+                lower.25.target = lower.25.target,
+                upper.125.target = upper.125.target,
+                lower.125.target = lower.125.target,
+                plot = FALSE
+            )$Certainty
+          )
+        }
+        #Certainty associated with samples
         NNS.ANOVA.rho <- mean(unlist(raw.certainties))
-
-  #Graphs
+        #Graphs
         if(plot){
-            boxplot(A, las = 2, xlab = "Means", ylab = "Variable", horizontal = TRUE, main = "NNS ANOVA", col = c('steelblue', rainbow(n - 1)))
-
-  #For ANOVA Visualization
+            boxplot(
+                A, 
+                las = 2, 
+                xlab = "Means", 
+                ylab = "Variable", 
+                horizontal = TRUE, 
+                main = "NNS ANOVA", 
+                col = c('steelblue', rainbow(n - 1))
+            )
+            #For ANOVA Visualization
             abline(v = mean.of.means, col = "red", lwd = 4)
             mtext("Grand Mean", side = 3,col = "red", at = mean.of.means)
         }
-
         return(c("Certainty" = NNS.ANOVA.rho))
-
-    } else {
-          n <- ncol(A)
-          raw.certainties <- list(n - 1)
-
-          for(i in 1:(n - 1)){
-              raw.certainties[[i]] <- sapply((i + 1) : n, function(b) NNS.ANOVA.bin(A[ , i],A[ , b], plot = FALSE)$Certainty)
-          }
-
-          certainties <- matrix(NA, n, n)
-          certainties[lower.tri(certainties, diag = FALSE)] <- unlist(raw.certainties)
-          diag(certainties) <- 1
-          certainties <- pmax(certainties, t(certainties), na.rm = TRUE)
-
-          colnames(certainties) <- colnames(A)
-          rownames(certainties) <- colnames(A)
-
-          if(plot){
-              boxplot(A, las = 2, xlab = "Means", ylab = "Variable", horizontal = TRUE, main = "ANOVA", col = c('steelblue', rainbow(n - 1)))
-              abline(v = mean.of.means, col = "red", lwd = 4)
-              mtext("Grand Mean", side = 3,col = "red", at = mean.of.means)
-          }
-
-          return(certainties)
-      }
+    }
+    raw.certainties <- list(n - 1)
+    for(i in 1:(n - 1)){
+      raw.certainties[[i]] <- sapply(
+        (i + 1) : n, 
+        function(b) NNS.ANOVA.bin(A[ , i],A[ , b], plot = FALSE)$Certainty
+      )
+    }
+    certainties <- matrix(NA, n, n)
+    certainties[lower.tri(certainties, diag = FALSE)] <- unlist(raw.certainties)
+    diag(certainties) <- 1
+    certainties <- pmax(certainties, t(certainties), na.rm = TRUE)
+    colnames(certainties) <- colnames(A)
+    rownames(certainties) <- colnames(A)
+    if(plot){
+      boxplot(
+        A, 
+        las = 2, 
+        xlab = "Means", 
+        ylab = "Variable", 
+        horizontal = TRUE, 
+        main = "ANOVA", 
+        col = c('steelblue', rainbow(n - 1))
+      )
+      abline(v = mean.of.means, col = "red", lwd = 4)
+      mtext("Grand Mean", side = 3,col = "red", at = mean.of.means)
+    }
+    return(certainties)
 }
