@@ -7,7 +7,6 @@
 #' @param continuous logical; \code{TRUE} (default) Generates a continuous measure using degree 1 \link{PM.matrix}, while discrete \code{FALSE} uses degree 0 \link{PM.matrix}.
 #' @param plot logical; \code{FALSE} (default) Generates a 3d scatter plot with regression points using \link{plot3d}.
 #' @param independence.overlay logical; \code{FALSE} (default) Creates and overlays independent \link{Co.LPM} and \link{Co.UPM} regions to visually reference the difference in dependence from the data.frame of variables being analyzed.  Under independence, the light green and red shaded areas would be occupied by green and red data points respectively.
-#' @param ncores integer; value specifying the number of cores to be used in the parallelized  procedure. If NULL (default), the number of cores to be used is equal to the number of cores of the machine - 1.
 #'
 #' @return Returns a multivariate dependence value [0,1].
 #'
@@ -17,21 +16,23 @@
 #' set.seed(123)
 #' x <- rnorm(1000) ; y <- rnorm(1000) ; z <- rnorm(1000)
 #' A <- data.frame(x, y, z)
-#' NNS.copula(A, plot = TRUE, independence.overlay = TRUE, ncores = 1)
+#' NNS.copula(A, target = colMeans(A), plot = TRUE, independence.overlay = TRUE)
 #'
 #' ### Target 0
-#' NNS.copula(A, target = rep(0, ncol(A)), plot = TRUE, independence.overlay = TRUE, ncores = 1)
+#' NNS.copula(A, target = rep(0, ncol(A)), plot = TRUE, independence.overlay = TRUE)
 #' @export
 
 
-NNS.copula <- function (X,
-                        target = NULL,
-                        continuous = TRUE,
-                        plot = FALSE,
-                        independence.overlay = FALSE,
-                        ncores = NULL){
-
-  if(sum(is.na(X)) > 0) stop("You have some missing values, please address.")
+NNS.copula <- function (
+  X,
+  target = NULL,
+  continuous = TRUE,
+  plot = FALSE,
+  independence.overlay = FALSE
+){
+  if(sum(is.na(X)) > 0){
+	stop("You have some missing values, please address.")
+  }
 
   n <- ncol(X)
   l <- dim(X)[1]
@@ -49,13 +50,13 @@ NNS.copula <- function (X,
   if(continuous) degree <- 1 else degree <- 0
 
   # Generate partial moment matrices
-  pm_cov <- PM.matrix(degree, degree, target = target, variable = X, pop.adj = TRUE, ncores = ncores)
+  pm_cov <- PM.matrix(degree, degree, target = target, variable = X, pop_adj = TRUE)
 
   # Isolate the upper triangles from each of the partial moment matrices
   Co_pm <- sum(pm_cov$cupm[upper.tri(pm_cov$cupm, diag = FALSE)]) + sum(pm_cov$clpm[upper.tri(pm_cov$clpm, diag = FALSE)])
   D_pm <- sum(pm_cov$dupm[upper.tri(pm_cov$dupm, diag = FALSE)]) + sum(pm_cov$dlpm[upper.tri(pm_cov$dlpm, diag = FALSE)])
 
-  if(plot && n == 3){
+  if((plot||independence.overlay) && n == 3){
 
     rgl::plot3d(x = X[ , 1], y = X[ , 2], z = X[ , 3], box = FALSE, size = 3,
                 col=ifelse((X[ , 1] <= mean(X[ , 1])) & (X[ , 2] <= mean(X[ , 2])) & (X[ , 3] <= mean(X[ , 3])), 'red' ,
