@@ -18,7 +18,7 @@ UPM(2, mean(x), x) + LPM(2, mean(x), x)
 (UPM(2, mean(x), x) + LPM(2, mean(x), x)) * (length(x) / (length(x) - 1))
 
 # Variance is also the co-variance of itself:
-(Co.LPM(1, 1, x, x, mean(x), mean(x)) + Co.UPM(1, 1, x, x, mean(x), mean(x)) - D.LPM(1, 1, x, x, mean(x), mean(x)) - D.UPM(1, 1, x, x, mean(x), mean(x))) * (length(x) / (length(x) - 1))
+(Co.LPM(1, x, x, mean(x), mean(x)) + Co.UPM(1, x, x, mean(x), mean(x)) - D.LPM(1, 1, x, x, mean(x), mean(x)) - D.UPM(1, 1, x, x, mean(x), mean(x))) * (length(x) / (length(x) - 1))
 
 ## ----stdev--------------------------------------------------------------------
 sd(x)
@@ -26,14 +26,17 @@ sd(x)
 
 ## ----covariance---------------------------------------------------------------
 cov(x, y)
-(Co.LPM(1, 1, x, y, mean(x), mean(y)) + Co.UPM(1, 1, x, y, mean(x), mean(y)) - D.LPM(1, 1, x, y, mean(x), mean(y)) - D.UPM(1, 1, x, y, mean(x), mean(y))) * (length(x) / (length(x) - 1))
+(Co.LPM(1, x, y, mean(x), mean(y)) + Co.UPM(1, x, y, mean(x), mean(y)) - D.LPM(1, 1, x, y, mean(x), mean(y)) - D.UPM(1, 1, x, y, mean(x), mean(y))) * (length(x) / (length(x) - 1))
 
-## ----cov_dec------------------------------------------------------------------
-PM.matrix(LPM.degree = 1, UPM.degree = 1,target = 'mean', variable = cbind(x, y), pop.adj = TRUE)
+## ----cov_dec, warning=FALSE---------------------------------------------------
+PM.matrix(LPM_degree = 1, UPM_degree = 1,target = 'mean', variable = cbind(x, y), pop_adj = TRUE)
+
+# Standard Covariance Matrix
+cov(cbind(x, y))
 
 ## ----pearson------------------------------------------------------------------
 cor(x, y)
-cov.xy = (Co.LPM(1, 1, x, y, mean(x), mean(y)) + Co.UPM(1, 1, x, y, mean(x), mean(y)) - D.LPM(1, 1, x, y, mean(x), mean(y)) - D.UPM(1, 1, x, y, mean(x), mean(y))) * (length(x) / (length(x) - 1))
+cov.xy = (Co.LPM(1, x, y, mean(x), mean(y)) + Co.UPM(1, x, y, mean(x), mean(y)) - D.LPM(1, 1, x, y, mean(x), mean(y)) - D.UPM(1, 1, x, y, mean(x), mean(y))) * (length(x) / (length(x) - 1))
 sd.x = ((UPM(2, mean(x), x) + LPM(2, mean(x), x)) * (length(x) / (length(x) - 1))) ^ .5
 sd.y = ((UPM(2, mean(y), y) + LPM(2, mean(y) , y)) * (length(y) / (length(y) - 1))) ^ .5
 cov.xy / (sd.x * sd.y)
@@ -51,10 +54,10 @@ points(sort(x), LPM(0, sort(x), x), col = "red")
 legend("left", legend = c("ecdf", "LPM.CDF"), fill = c("black", "red"), border = NA, bty = "n")
 
 # Joint CDF:
-Co.LPM(0, 0, x, y, 0, 0)
+Co.LPM(0, x, y, 0, 0)
 
 # Vectorized targets:
-Co.LPM(0, 0, x, y, c(0, 1), c(0, 1))
+Co.LPM(0, x, y, c(0, 1), c(0, 1))
 
 # Continuous CDF:
 NNS.CDF(x, 1)
